# Enthought library imports.
from traits.api import Array, Interface, Str


class IModel2d(Interface):

    # The user-visible name of the model.
    name = Str()

    x_data = Array()
    y_data = Array()



# (C) Copyright 2007-2022 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!
""" An  IPython kernel plugin. """

# Enthought library imports.
from envisage.plugins.ipython_kernel.ipython_kernel_plugin import (
    IPYTHON_KERNEL_PROTOCOL,
)
from envisage.ui.tasks.api import TaskExtension
from envisage.api import Plugin
from traits.api import List
from pyface.tasks.action.api import SGroup, SchemaAddition


TASK_EXTENSIONS = "envisage.ui.tasks.task_extensions"


class IPythonKernelUIPlugin(Plugin):
    """ Contributes UI actions on top of the IPython Kernel Plugin. """

    #### 'IPlugin' interface ##################################################

    #: The plugin unique identifier.
    id = "envisage.plugins.ipython_kernel_ui"

    #: The plugin name (suitable for displaying to the user).
    name = "IPython embedded kernel UI plugin"

    #### Contributions to extension points made by this plugin ################

    contributed_task_extensions = List(contributes_to=TASK_EXTENSIONS)

    #### Trait initializers ###################################################

    def _contributed_task_extensions_default(self):

        from .actions import StartQtConsoleAction

        def menu_factory():
            kernel = self.application.get_service(IPYTHON_KERNEL_PROTOCOL)
            return SGroup(StartQtConsoleAction(kernel=kernel), id="ipython")

        return [
            TaskExtension(
                actions=[
                    SchemaAddition(
                        path="MenuBar/View",
                        factory=menu_factory,
                        id="IPythonSchema",
                    ),
                ]
            )
        ]



# Standard library imports.
from os.path import basename

# Major package imports.
from pyface.qt import QtCore, QtGui

# Enthought library imports.
from traits.api import (
    Bool, Event, Instance, File, observe, Property, provides, Str,
)
from pyface.tasks.api import Editor

# Local imports.
from i_python_editor import IPythonEditor
from pyface.key_pressed_event import KeyPressedEvent


@provides(IPythonEditor)
class PythonEditor(Editor):
    """ The toolkit specific implementation of a PythonEditor.  See the
    IPythonEditor interface for the API documentation.
    """

    #### 'IPythonEditor' interface ############################################

    obj = Instance(File)

    path = Str

    dirty = Bool(False)

    name = Property(Str, observe="path")

    tooltip = Property(Str, observe="path")

    show_line_numbers = Bool(True)

    #### Events ####

    changed = Event

    key_pressed = Event(KeyPressedEvent)

    def _get_tooltip(self):
        return self.path

    def _get_name(self):
        return basename(self.path) or "Untitled"

    ###########################################################################
    # 'PythonEditor' interface.
    ###########################################################################

    def create(self, parent):
        self.control = self._create_control(parent)

    def load(self, path=None):
        """ Loads the contents of the editor.
        """
        if path is None:
            path = self.path

        # We will have no path for a new script.
        if len(path) > 0:
            with open(self.path, "r", encoding="utf-8") as f:
                text = f.read()
        else:
            text = ""

        self.control.code.setPlainText(text)
        self.dirty = False

    def save(self, path=None):
        """ Saves the contents of the editor.
        """
        if path is None:
            path = self.path

        f = file(path, "w")
        f.write(self.control.code.toPlainText())
        f.close()

        self.dirty = False

    def select_line(self, lineno):
        """ Selects the specified line.
        """
        self.control.code.set_line_column(lineno, 0)
        self.control.code.moveCursor(
            QtGui.QTextCursor.EndOfLine, QtGui.QTextCursor.KeepAnchor
        )

    ###########################################################################
    # Trait handlers.
    ###########################################################################

    @observe("path")
    def _reload_data(self, event):
        if self.control is not None:
            self.load()

    @observe("show_line_numbers")
    def _update_visible_lines(self, event=None):
        if self.control is not None:
            self.control.code.line_number_widget.setVisible(
                self.show_line_numbers
            )
            self.control.code.update_line_number_width()

    ###########################################################################
    # Private interface.
    ###########################################################################

    def _create_control(self, parent):
        """ Creates the toolkit-specific control for the widget.
        """
        from pyface.ui.qt4.code_editor.code_widget import AdvancedCodeWidget

        self.control = control = AdvancedCodeWidget(parent)
        self._update_visible_lines()

        # Install event filter to trap key presses.
        event_filter = PythonEditorEventFilter(self, self.control)
        self.control.installEventFilter(event_filter)
        self.control.code.installEventFilter(event_filter)

        # Connect signals for text changes.
        control.code.modificationChanged.connect(self._toggle_dirty)
        control.code.textChanged.connect(self._toggle_changed_trait)

        # Load the editor's contents.
        self.load()

        return control

    def _toggle_dirty(self, dirty):
        """ Called whenever a change is made to the dirty state of the
            document.
        """
        self.dirty = dirty

    def _toggle_changed_trait(self):
        """ Called whenever a change is made to the text of the document.
        """
        self.changed = True


class PythonEditorEventFilter(QtCore.QObject):
    """ A thin wrapper around the advanced code widget to handle the
    key_pressed Event.
    """

    def __init__(self, editor, parent):
        super().__init__(parent)
        self.__editor = editor

    def eventFilter(self, obj, event):
        """ Reimplemented to trap key presses.
        """
        if (
            self.__editor.control
            and obj == self.__editor.control
            and event.type() == QtCore.QEvent.FocusOut
        ):
            # Hack for Traits UI compatibility.
            self.__editor.control.emit(QtCore.SIGNAL("lostFocus"))

        elif (
            self.__editor.control
            and obj == self.__editor.control.code
            and event.type() == QtCore.QEvent.KeyPress
        ):
            # Pyface doesn't seem to be Unicode aware.  Only keep the key code
            # if it corresponds to a single Latin1 character.
            kstr = event.text()
            try:
                kcode = ord(str(kstr))
            except:
                kcode = 0

            mods = event.modifiers()
            self.key_pressed = KeyPressedEvent(
                alt_down=(
                    (mods & QtCore.Qt.AltModifier) == QtCore.Qt.AltModifier
                ),
                control_down=(
                    (mods & QtCore.Qt.ControlModifier)
                    == QtCore.Qt.ControlModifier
                ),
                shift_down=(
                    (mods & QtCore.Qt.ShiftModifier) == QtCore.Qt.ShiftModifier
                ),
                key_code=kcode,
                event=event,
            )

        return super().eventFilter(obj, event)



# Major package imports.
from setuptools import setup, find_packages

setup(
    name="acme.workbench",
    version="0.1a1",
    author="Enthought, Inc",
    author_email="info@enthought.com",
    license="BSD",
    zip_safe=False,
    packages=find_packages(),
    include_package_data=True,
    namespace_packages=["acme", "acme.workbench"],
    entry_points="""
    [envisage.plugins]
    acme_workbench = acme.workbench.acme_workbench_plugin:AcmeWorkbenchPlugin
    """,
)



""" The preferences for the Acme workbench. """


# Enthought library imports.
from apptools.preferences.ui.api import PreferencesPage
from traits.api import Bool, Color, Int, Float, Font, Str
from traitsui.api import View


class AcmePreferencesPage(PreferencesPage):
    """ The preferences page for the Acme workbench. """

    #### 'PreferencesPage' interface ##########################################

    # The page's category (e.g. 'General/Appearance'). The empty string means
    # that this is a top-level page.
    category = "General"

    # The page's help identifier (optional). If a help Id *is* provided then
    # there will be a 'Help' button shown on the preference page.
    help_id = ""

    # The page name (this is what is shown in the preferences dialog.
    name = "Acme"

    # The path to the preference node that contains the preferences.
    preferences_path = "acme.workbench"

    #### Preferences ##########################################################

    # Width.
    width = Int(100)

    # Height.
    height = Int(200)

    # Ratio.
    ratio = Float(0.1)

    # Background color.
    bgcolor = Color("red")

    # Text font.
    font = Font("helvetica")

    #### Traits UI views ######################################################

    trait_view = View("width", "height", "ratio", "font", "bgcolor")



# (C) Copyright 2007-2022 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" An IPython kernel plugin. """

import logging
import warnings

# Enthought library imports.
from envisage.api import (
    bind_extension_point,
    ExtensionPoint,
    Plugin,
    ServiceOffer,
)
from traits.api import Bool, Instance, List

# Constants kept around for backwards compatibility.
# These will be removed in a future release.
# Extension point IDs.
SERVICE_OFFERS = "envisage.service_offers"
IPYTHON_NAMESPACE = "ipython_plugin.namespace"

# Protocol for the contributed service offer.
IPYTHON_KERNEL_PROTOCOL = (
    "envisage.plugins.ipython_kernel.internal_ipkernel.InternalIPKernel")

logger = logging.getLogger(__name__)


class IPythonKernelPlugin(Plugin):
    """ An IPython kernel plugin. """

    #: The plugin unique identifier.
    id = "envisage.plugins.ipython_kernel"

    #: The plugin name (suitable for displaying to the user).
    name = "IPython embedded kernel plugin"

    #: Extension point for objects contributed to the IPython kernel namespace.
    IPYTHON_NAMESPACE = 'ipython_plugin.namespace'

    #: Ipython kernel namespace ExtensionPoint
    kernel_namespace = ExtensionPoint(
        List,
        id=IPYTHON_NAMESPACE,
        desc="""

        Variables to add to the IPython kernel namespace.
        This is a list of tuples (name, value).

        """,
    )

    #: Service offers contributed by this plugin.
    service_offers = List(contributes_to=SERVICE_OFFERS)

    # Protocol for the contributed service offer.
    IPYTHON_KERNEL_PROTOCOL = 'envisage.plugins.ipython_kernel.internal_ipkernel.InternalIPKernel'  # noqa: E501

    #: Whether to initialize the kernel when the service is created.
    #: The default is ``False```, for backwards compatibility. It will change
    #: to ``True`` in a future version of Envisage. External users wanting
    #: to use the future behaviour now should pass ``init_ipkernel=True``
    #: when creating the plugin.
    init_ipkernel = Bool(False)

    def stop(self):
        """ Stop the plugin. """
        self._destroy_kernel()

    # Private traits and methods

    #: The InternalIPKernel instance provided by the service.
    _kernel = Instance(IPYTHON_KERNEL_PROTOCOL)

    def _create_kernel(self):
        from .internal_ipkernel import InternalIPKernel

        # This shouldn't happen with a normal lifecycle, but add a warning
        # just in case.
        if self._kernel is not None:
            warnings.warn(
                "A kernel already exists. " "No new kernel will be created.",
                RuntimeWarning,
            )
            return

        logger.debug("Creating the embedded IPython kernel")
        kernel = self._kernel = InternalIPKernel()
        bind_extension_point(
            kernel, "initial_namespace", IPYTHON_NAMESPACE, self.application
        )
        if self.init_ipkernel:
            kernel.init_ipkernel()
        else:
            warnings.warn(
                (
                    "In the future, the IPython kernel will be initialized "
                    "automatically at creation time. To enable this "
                    "future behaviour now, create the plugin using "
                    "IPythonKernelPlugin(init_ipkernel=True)"
                ),
                DeprecationWarning,
            )

        return kernel

    def _destroy_kernel(self):
        """
        Destroy any existing kernel.
        """
        if self._kernel is None:
            return

        logger.debug("Shutting down the embedded IPython kernel")
        self._kernel.shutdown()
        self._kernel = None

    def _service_offers_default(self):
        ipython_kernel_service_offer = ServiceOffer(
            protocol=IPYTHON_KERNEL_PROTOCOL, factory=self._create_kernel,
        )
        return [ipython_kernel_service_offer]



# (C) Copyright 2007-2022 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

import atexit
import gc
import os
import shutil
import sys
import tempfile
import threading
import unittest
from unittest import mock
import warnings

# Envisage is not currently compatible with ipykernel >= 6 or IPython >= 8. See
# enthought/envisage#448.
try:
    import ipykernel
    import IPython
except ImportError:
    ipykernel_available = False
else:
    ipykernel_available = (
        ipykernel.version_info < (6,)
        and IPython.version_info < (8,)
    )

if ipykernel_available:
    import ipykernel.iostream
    import ipykernel.ipkernel
    import ipykernel.kernelapp
    import ipykernel.zmqshell
    import IPython.utils.io
    import tornado.ioloop
    import zmq

    from envisage.plugins.ipython_kernel.api import InternalIPKernel


# In rare cases, isinstance(obj, type) can raise. An example occurs in Python
# 3.6 when obj has type `functools._lru_list_elem` and type is zmq.Socket. This
# particular example is fixed in Python >= 3.7
#
# It may be possible to replace uses of safe_isinstance with plain old
# isinstance once Python 3.6 support is no longer needed.
# xref: enthought/envisage#469

def safe_isinstance(obj, type):
    """
    Same as isinstance(obj, type), but suppresses AttributeError.
    """
    try:
        return isinstance(obj, type)
    except AttributeError:
        return False


@unittest.skipUnless(
    ipykernel_available, "skipping tests that require the ipykernel package"
)
class TestInternalIPKernel(unittest.TestCase):
    def setUp(self):
        # Make sure that IPython-related files are written to a temporary
        # directory instead of the home directory.
        tmpdir = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, tmpdir)

        self._old_ipythondir = os.environ.get("IPYTHONDIR")
        os.environ["IPYTHONDIR"] = tmpdir

    def tearDown(self):
        # Restore previous state of the IPYTHONDIR environment variable.
        old_ipythondir = self._old_ipythondir
        if old_ipythondir is None:
            del os.environ["IPYTHONDIR"]
        else:
            os.environ["IPYTHONDIR"] = old_ipythondir

    def test_lifecycle(self):
        kernel = InternalIPKernel()
        self.assertIsNone(kernel.ipkernel)

        kernel.init_ipkernel(gui_backend=None)
        self.assertIsNotNone(kernel.ipkernel)
        self.assertIsInstance(kernel.ipkernel, ipykernel.kernelapp.IPKernelApp)

        kernel.new_qt_console()
        kernel.new_qt_console()
        self.assertEqual(len(kernel.consoles), 2)

        kernel.shutdown()
        self.assertIsNone(kernel.ipkernel)
        self.assertEqual(len(kernel.consoles), 0)

    def test_initial_namespace(self):
        kernel = InternalIPKernel(initial_namespace=[("x", 42.1)])
        kernel.init_ipkernel(gui_backend=None)
        self.assertIn("x", kernel.namespace)
        self.assertEqual(kernel.namespace["x"], 42.1)
        kernel.shutdown()

    def test_shutdown_restores_output_streams(self):
        original_stdin = sys.stdin
        original_stdout = sys.stdout
        original_stderr = sys.stderr

        self.create_and_destroy_kernel()

        self.assertIs(sys.stdin, original_stdin)
        self.assertIs(sys.stdout, original_stdout)
        self.assertIs(sys.stderr, original_stderr)

    def test_shutdown_restores_sys_modules_main(self):
        original_sys_modules_main = sys.modules["__main__"]
        self.create_and_destroy_kernel()
        self.assertIs(sys.modules["__main__"], original_sys_modules_main)

    def test_shutdown_restores_displayhook_and_excepthook(self):
        original_displayhook = sys.displayhook
        original_excepthook = sys.excepthook

        self.create_and_destroy_kernel()

        self.assertIs(sys.displayhook, original_displayhook)
        self.assertIs(sys.excepthook, original_excepthook)

    def test_shutdown_restores_sys_path(self):
        original_sys_path = sys.path[:]

        self.create_and_destroy_kernel()

        self.assertEqual(sys.path, original_sys_path)

    def test_shutdown_closes_console_pipes(self):
        kernel = InternalIPKernel()
        kernel.init_ipkernel(gui_backend=None)
        console = kernel.new_qt_console()
        self.assertFalse(console.stdout.closed)
        self.assertFalse(console.stderr.closed)
        kernel.shutdown()
        self.assertTrue(console.stdout.closed)
        self.assertTrue(console.stderr.closed)

    def test_ipython_util_io_globals_restored(self):
        original_io_stdin = IPython.utils.io.stdin
        original_io_stdout = IPython.utils.io.stdout
        original_io_stderr = IPython.utils.io.stderr

        self.create_and_destroy_kernel()

        self.assertIs(IPython.utils.io.stdin, original_io_stdin)
        self.assertIs(IPython.utils.io.stdout, original_io_stdout)
        self.assertIs(IPython.utils.io.stderr, original_io_stderr)

    def test_ipython_util_io_globals_restored_if_they_dont_exist(self):
        # Regression test for enthought/envisage#218
        original_io_stdin = IPython.utils.io.stdin
        original_io_stdout = IPython.utils.io.stdout
        original_io_stderr = IPython.utils.io.stderr

        del IPython.utils.io.stdin
        del IPython.utils.io.stdout
        del IPython.utils.io.stderr

        try:
            self.create_and_destroy_kernel()
            self.assertFalse(hasattr(IPython.utils.io, "stdin"))
            self.assertFalse(hasattr(IPython.utils.io, "stdout"))
            self.assertFalse(hasattr(IPython.utils.io, "stderr"))
        finally:
            IPython.utils.io.stdin = original_io_stdin
            IPython.utils.io.stdout = original_io_stdout
            IPython.utils.io.stderr = original_io_stderr

    def test_io_pub_thread_stopped(self):
        self.create_and_destroy_kernel()
        io_pub_threads = self.objects_of_type(ipykernel.iostream.IOPubThread)
        for thread in io_pub_threads:
            self.assertFalse(thread.thread.is_alive())

    def test_no_threads_leaked(self):
        threads_before = threading.active_count()
        self.create_and_destroy_kernel()
        threads_after = threading.active_count()
        self.assertEqual(threads_before, threads_after)

    def test_no_new_atexit_handlers(self):
        # Caution: this is a rather fragile and indirect test. We want
        # to know that all cleanup has happened when shutting down the
        # kernel, with none of that cleanup deferred to atexit handlers.
        #
        # Since we have no direct way to get hold of the atexit handlers on
        # Python 3, we instead use the number of referents from the
        # atexit module as a proxy.
        #
        # If this test starts failing, try adding a warmup cycle. If the
        # first call to self.create_and_destroy_kernel adds new referents,
        # that's not a big deal. But if every call consistently adds new
        # referents, then there's something to be fixed.
        atexit_handlers_before = len(gc.get_referents(atexit))
        self.create_and_destroy_kernel()
        atexit_handlers_after = len(gc.get_referents(atexit))

        self.assertEqual(atexit_handlers_before, atexit_handlers_after)

    def test_zmq_sockets_closed(self):
        # Previously, tests were leaking file descriptors linked to
        # zmq.Socket objects. Check that all extant sockets are closed.
        self.create_and_destroy_kernel()
        sockets = self.objects_of_type(zmq.Socket)
        self.assertTrue(all(socket.closed for socket in sockets))

    @unittest.skipIf(
        sys.platform == "win32" and sys.version_info[:2] == (3, 8),
        "xref: enthought/envisage#463"
    )
    def test_ipykernel_live_objects(self):
        # Check that all IPython-related objects have been cleaned up
        # as expected.

        self.create_and_destroy_kernel()

        # Remove anything kept alive by cycles. (There are too many cycles
        # to break them individually.)
        gc.collect()

        shells = self.objects_of_type(ipykernel.zmqshell.ZMQInteractiveShell)
        self.assertEqual(shells, [])

        kernels = self.objects_of_type(ipykernel.ipkernel.IPythonKernel)
        self.assertEqual(kernels, [])

        kernel_apps = self.objects_of_type(ipykernel.kernelapp.IPKernelApp)
        self.assertEqual(kernel_apps, [])

    def test_initialize_twice(self):
        # Trying to re-initialize an already initialized IPKernelApp can
        # happen right now as a result of refactoring, but eventually
        # it should be an error. For now, it's a warning.
        kernel = InternalIPKernel()
        self.assertIsNone(kernel.ipkernel)
        kernel.init_ipkernel(gui_backend=None)
        try:
            self.assertIsNotNone(kernel.ipkernel)
            ipkernel = kernel.ipkernel

            with warnings.catch_warnings(record=True) as warn_msgs:
                warnings.simplefilter("always", category=DeprecationWarning)
                kernel.init_ipkernel(gui_backend=None)

            # Check that the existing kernel has not been replaced.
            self.assertIs(ipkernel, kernel.ipkernel)
        finally:
            kernel.shutdown()

        # Check that we got the expected warning message.
        self.assertEqual(len(warn_msgs), 1)
        message = str(warn_msgs[0].message)
        self.assertIn("already been initialized", message)

    def test_init_ipkernel_with_explicit_gui_backend(self):
        loop = tornado.ioloop.IOLoop.current()

        # Kernel initialization adds an "enter_eventloop" call to the
        # ioloop event loop queue. Mock to avoid modifying the actual event
        # loop.
        with mock.patch.object(loop, "add_callback") as mock_add_callback:
            with warnings.catch_warnings(record=True) as warn_msgs:
                warnings.simplefilter("always", category=DeprecationWarning)

                # Use of gui_backend is deprecated.
                kernel = InternalIPKernel()
                kernel.init_ipkernel(gui_backend="qt4")
                kernel.shutdown()

        mock_add_callback.reset_mock()

        # Check that we got the expected warning message.
        matching_messages = [
            msg
            for msg in warn_msgs
            if "gui_backend argument is deprecated" in str(msg.message)
        ]
        self.assertEqual(len(matching_messages), 1)

    # Helper functions.

    def objects_of_type(self, type):
        """
        Find and return a list of all currently tracked instances of the
        given type.
        """
        return [obj for obj in gc.get_objects() if safe_isinstance(obj, type)]

    def create_and_destroy_kernel(self):
        """
        Set up a new kernel with two associated consoles, then shut everything
        down.
        """
        kernel = InternalIPKernel()
        kernel.init_ipkernel(gui_backend=None)
        kernel.new_qt_console()
        kernel.new_qt_console()
        kernel.shutdown()



""" An example perspective. """


# Enthought library imports.
from pyface.workbench.api import Perspective, PerspectiveItem


class BarPerspective(Perspective):
    """ An example perspective. """

    # The perspective's name.
    name = "Bar"

    # Should the editor area be shown in this perspective?
    show_editor_area = False

    # The contents of the perspective.
    contents = [
        PerspectiveItem(id="Green"),
        PerspectiveItem(id="Black", position="bottom", relative_to="Green"),
    ]



""" An example perspective. """


# Enthought library imports.
from pyface.workbench.api import Perspective, PerspectiveItem


class FooPerspective(Perspective):
    """ An example perspective. """

    # The perspective's name.
    name = "Foo"

    # Should the editor area be shown in this perspective?
    show_editor_area = True

    # The contents of the perspective.
    contents = [
        PerspectiveItem(id="Blue", position="left"),
        PerspectiveItem(id="Red", position="with", relative_to="Blue"),
        PerspectiveItem(id="Green", position="top"),
    ]



# Plugin imports.
from envisage.api import CorePlugin
from envisage.ui.tasks.api import TasksPlugin


def main(argv):
    """ Run the application.
    """
    # Import here so that this script can be run from anywhere without
    # having to install the packages.
    from attractors.attractors_plugin import AttractorsPlugin
    from attractors.attractors_application import AttractorsApplication

    plugins = [CorePlugin(), TasksPlugin(), AttractorsPlugin()]
    app = AttractorsApplication(plugins=plugins)
    app.run()


if __name__ == "__main__":
    import sys
    # This context manager is added so that one can run this example from any
    # directory without necessarily having installed the examples as packages.
    from envisage.examples._demo import demo_path

    with demo_path(__file__):
        main(sys.argv)



""" The Lorenz UI plugin. """


# Enthought library imports.
from envisage.api import Plugin
from pyface.workbench.api import Perspective, PerspectiveItem
from pyface.workbench.api import TraitsUIView
from traits.api import List


class LorenzPerspective(Perspective):
    """ A perspective containing the default Lorenz views. """

    name = "Lorenz"
    show_editor_area = False

    contents = [
        PerspectiveItem(id="lorenz.data"),
        PerspectiveItem(id="lorenz.plot2d"),
    ]


class LorenzUIPlugin(Plugin):
    """ The Lorenz UI plugin.

    This plugin is part of the 'Lorenz' example application.

    """

    # Extension points Ids.
    PERSPECTIVES = "envisage.ui.workbench.perspectives"
    VIEWS = "envisage.ui.workbench.views"

    #### 'IPlugin' interface ##################################################

    # The plugin's unique identifier.
    id = "acme.lorenz.ui"

    # The plugin's name (suitable for displaying to the user).
    name = "Lorenz UI"

    #### Contributions to extension points made by this plugin ################

    # Perspectives.
    perspectives = List(contributes_to=PERSPECTIVES)

    def _perspectives_default(self):
        """ Trait initializer. """

        return [LorenzPerspective]

    # Views.
    views = List(contributes_to=VIEWS)

    def _views_default(self):
        """ Trait initializer. """

        return [self._create_data_view, self._create_plot2d_view]

    ###########################################################################
    # Private interface.
    ###########################################################################

    def _create_data_view(self, **traits):
        """ Factory method for the data view. """

        from acme.lorenz.api import DataView, Lorenz

        data_view = TraitsUIView(
            id="lorenz.data",
            name="Data",
            obj=DataView(lorenz=self.application.get_service(Lorenz)),
            **traits,
        )

        return data_view

    def _create_plot2d_view(self, **traits):
        """ Factory method for the plot2D view. """

        from acme.lorenz.api import Lorenz, Plot2DView

        plot2d_view = TraitsUIView(
            id="lorenz.plot2d",
            name="Plot 2D",
            obj=Plot2DView(lorenz=self.application.get_service(Lorenz)),
            **traits,
        )

        return plot2d_view



# Enthought library imports.
from envisage.api import CorePlugin, Plugin
from envisage.plugins.ipython_kernel.api import (
    IPythonKernelPlugin,
    IPYTHON_KERNEL_PROTOCOL,
    IPythonKernelUIPlugin,
)
from envisage.ui.tasks.api import TasksApplication, TaskFactory, TasksPlugin
from pyface.qt import QtCore
from pyface.tasks.api import TaskWindowLayout
from pyface.util.guisupport import get_app_qt4
from traits.api import List

# Local imports
from example_task import ExampleTask


class ExamplePlugin(Plugin):

    #### 'IPlugin' interface ##############################################

    # The plugin's unique identifier.
    id = "example.plugins.ipython_kernel"

    # The plugin's name (suitable for displaying to the user).
    name = "IPython Kernel Example Plugin"

    #### Contributions to extension points made by this plugin ############

    kernel_namespace = List(contributes_to="ipython_plugin.namespace")

    def _kernel_namespace_default(self):
        namespace = [("app", self.application)]
        return namespace

    tasks = List(contributes_to="envisage.ui.tasks.tasks")

    def _tasks_default(self):
        print("Default tasks")
        return [
            TaskFactory(
                id="example_task", name="Example task", factory=ExampleTask
            )
        ]


class ExampleApplication(TasksApplication):

    #### 'IApplication' interface #########################################

    # The application's globally unique identifier.
    id = "example.ipython_kernel"

    # The application's user-visible name.
    name = "Example app"

    always_use_default_layout = True

    #### 'TasksApplication' interface #####################################

    # The default application-level layout for the application.
    default_layout = [TaskWindowLayout("example_task", size=(800, 600))]

    def run(self):
        """ Run the application.

        Returns
        -------
        bool
            Whether the application started successfully (i.e., without a
            veto).
        """
        # Make sure the GUI has been created (so that, if required, the splash
        # screen is shown).
        gui = self.gui

        started = self.start()
        if started:
            app = get_app_qt4([""])

            # Create windows from the default or saved application layout.
            self._create_windows()

            kernel = self.get_service(IPYTHON_KERNEL_PROTOCOL)
            kernel.init_ipkernel("qt4")

            app.connect(
                app,
                QtCore.SIGNAL("lastWindowClosed()"),
                app,
                QtCore.SLOT("quit()"),
            )
            app.aboutToQuit.connect(kernel.cleanup_consoles)

            gui.set_trait_later(self, "application_initialized", self)
            kernel.ipkernel.start()

        return started


if __name__ == "__main__":

    app = ExampleApplication(
        plugins=[
            CorePlugin(),
            ExamplePlugin(),
            IPythonKernelPlugin(),
            IPythonKernelUIPlugin(),
            TasksPlugin(),
        ]
    )
    app.run()



""" An action that dynamically creates and adds a view. """


# Enthought library imports.
from pyface.api import ImageResource
from pyface.action.api import Action
from pyface.workbench.api import View


class NewViewAction(Action):
    """ An action that dynamically creates and adds a view. """

    #### 'Action' interface ###################################################

    # A longer description of the action.
    description = "Create and add a new view"

    # The action's name (displayed on menus/tool bar tools etc).
    name = "New View"

    # A short description of the action used for tooltip text etc.
    tooltip = "Create and add a new view"

    ###########################################################################
    # 'Action' interface.
    ###########################################################################

    def perform(self, event):
        """ Perform the action. """

        # You can give the view a position... (it default to 'left')...
        view = View(id="my.view.fred", name="Fred", position="right")
        self.window.add_view(view)

        # or you can specify it on the call to 'add_view'...
        view = View(id="my.view.wilma", name="Wilma")
        self.window.add_view(view, position="top")

        return



""" Lorenz example. """


# Major package imports.
from scipy import array, arange
from scipy.integrate import odeint

# Enthought library imports.
from chaco.chaco_plot_editor import ChacoPlotItem
from traits.api import Array, DelegatesTo, Float, HasTraits, observe
from traits.api import Instance, List, Trait
from traitsui.api import Item, HGroup, VGroup, View


class Lorenz(HasTraits):
    """ The Lorenz model. """

    prandtl = Float(10.0, auto_set=False, enter_set=True)
    rayleigh = Float(28.0, auto_set=False, enter_set=True)
    beta = Float(8.0 / 3.0, auto_set=False, enter_set=True)
    # Give the dtype explicitly in the Array traits; this is a
    # work-around for trac ticket #1864.
    init = Array(value=array([0.0, 1.0, 0.0]), dtype=float)
    time = Array(value=array([0.0, 100.0, 0.01]), dtype=float)
    timePoints = Array()
    data3d = Array()
    output = Trait(
        "x vs time", {"x vs time": 0, "y vs time": 1, "z vs time": 2}
    )
    data2d = Array()

    def refresh(self):
        self.calculatePoints()
        self.data2d = self.data3d[:, self.output_]

    def __init__(self):
        self.refresh()

    @observe("output,prandtl,rayleigh,beta,init,time")
    def _refresh_model_data(self, event):
        self.refresh()

    def lorenz(self, w, t, prandtl, rayleigh, beta):
        x, y, z = w
        return array(
            [prandtl * (y - x), x * (rayleigh - z) - y, x * y - beta * z]
        )

    def calculatePoints(self):
        init = self.init.copy()
        self.timePoints = arange(*self.time)
        self.data3d = odeint(
            self.lorenz,
            init,
            self.timePoints,
            args=(self.prandtl, self.rayleigh, self.beta),
        )

        return


class DataView(HasTraits):
    """ The data view. """

    # The model that we are a view of.
    lorenz = Instance(Lorenz)

    # The view traits.
    prandtl = DelegatesTo("lorenz")
    rayleigh = DelegatesTo("lorenz")
    beta = DelegatesTo("lorenz")
    init = DelegatesTo("lorenz")
    time = DelegatesTo("lorenz")

    traits_ui_view = View(
        Item("prandtl"),
        Item("rayleigh"),
        Item("beta"),
        Item("init"),
        Item("time"),
        id="lorenz.data",
        resizable=True,
    )


class Plot2DView(HasTraits):
    """ The Plot 2D view. """

    # The model that we are a view of.
    lorenz = Instance(Lorenz)

    # The view traits.
    output = DelegatesTo("lorenz")
    timePoints = DelegatesTo("lorenz")
    data2d = DelegatesTo("lorenz")

    traits_ui_view = View(
        Item("output"),
        ChacoPlotItem(
            "timePoints",
            "data2d",
            show_label=False,
            resizable=True,
            orientation="h",
            title="Plot",
            x_label="time",
            y_label="x",
            color="red",
            bgcolor="white",
            border_visible=False,
            border_width=1,
            padding_bg_color="lightgray",
        ),
        id="lorenz.plot2d",
        resizable=True,
    )



""" The Lorenz plugin. """


# Enthought library imports.
from envisage.api import Plugin, ServiceOffer
from traits.api import List


class LorenzPlugin(Plugin):
    """ The Lorenz plugin.

    This plugin is part of the 'Lorenz' example application.

    """

    # Extension points Ids.
    SERVICE_OFFERS = "envisage.service_offers"

    #### 'IPlugin' interface ##################################################

    # The plugin's unique identifier.
    id = "acme.lorenz"

    # The plugin's name (suitable for displaying to the user).
    name = "Lorenz"

    #### Contributions to extension points made by this plugin ################

    # Service offers.
    service_offers = List(contributes_to=SERVICE_OFFERS)

    def _service_offers_default(self):
        """ Trait initializer. """

        lorenz_service_offer = ServiceOffer(
            protocol="acme.lorenz.lorenz.Lorenz",
            factory="acme.lorenz.lorenz.Lorenz",
        )

        return [lorenz_service_offer]



# (C) Copyright 2007-2022 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!
from .i_resource_protocol import IResourceProtocol
from .i_resource_manager import IResourceManager

from .file_resource_protocol import FileResourceProtocol
from .http_resource_protocol import HTTPResourceProtocol
from .no_such_resource_error import NoSuchResourceError
from .package_resource_protocol import PackageResourceProtocol
from .resource_manager import ResourceManager



# Standard library imports.
import os.path

# Enthought library imports.
from pyface.tasks.api import TraitsDockPane
from traits.api import Event, File, List, Str
from traitsui.api import View, Item, FileEditor


class FileBrowserPane(TraitsDockPane):
    """ A simple file browser pane.
    """

    #### TaskPane interface ###################################################

    id = "example.file_browser_pane"
    name = "File Browser"

    #### FileBrowserPane interface ############################################

    # Fired when a file is double-clicked.
    activated = Event

    # The list of wildcard filters for filenames.
    filters = List(Str)

    # The currently selected file.
    selected_file = File(os.path.expanduser("~"))

    # The view used to construct the dock pane's widget.
    view = View(
        Item(
            "selected_file",
            editor=FileEditor(dclick_name="activated", filter_name="filters"),
            style="custom",
            show_label=False,
        ),
        resizable=True,
    )


class PythonScriptBrowserPane(FileBrowserPane):
    """ A file browser pane restricted to Python scripts.
    """

    #### TaskPane interface ###################################################

    id = "example.python_script_browser_pane"
    name = "Script Browser"

    #### FileBrowserPane interface ############################################

    filters = ["*.py"]



""" The Lorenz example application. """


# Standard library imports.
from logging import DEBUG

# Enthought library imports.
from envisage.ui.workbench.api import WorkbenchApplication
from pyface.api import AboutDialog, ImageResource, SplashScreen


class LorenzApplication(WorkbenchApplication):
    """ The Lorenz example application. """

    #### 'IApplication' interface #############################################

    # The application's globally unique Id.
    id = "acme.lorenz"

    #### 'WorkbenchApplication' interface #####################################

    # Branding information.
    #
    # The icon used on window title bars etc.
    icon = ImageResource("lorenz.ico")

    # The name of the application (also used on window title bars etc).
    name = "Lorenz"

    ###########################################################################
    # 'WorkbenchApplication' interface.
    ###########################################################################

    def _about_dialog_default(self):
        """ Trait initializer. """

        about_dialog = AboutDialog(
            parent=self.workbench.active_window.control,
            image=ImageResource("about"),
        )

        return about_dialog

    def _splash_screen_default(self):
        """ Trait initializer. """

        splash_screen = SplashScreen(
            image=ImageResource("splash"),
            show_log_messages=True,
            log_level=DEBUG,
        )

        return splash_screen



""" The interface for 'Message of the Day' messages. """


# Enthought library imports.
from traits.api import Interface, Str


class IMessage(Interface):
    """ The interface for 'Message of the Day' messages. """

    # The author of the message.
    author = Str

    # The text of the message.
    text = Str



# (C) Copyright 2007-2022 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!
""" Base class for Egg-based test cases. """


from os.path import join
import shutil
import subprocess
import sys
import tempfile
import unittest

import pkg_resources


def build_egg(egg_dir, dist_dir):
    """Helper function to build an egg.

    Parameters
    ----------
    egg_dir : str
        Directory containing the Python package to be built. Should
        contain a "setup.py" file that can be used with
        "python setup.py bdist_egg" to build the package.
    dist_dir : str
        Directory to place the built egg in. The directory should
        already exist.
    """
    subprocess.run(
        [
            "python",
            "setup.py",
            "bdist_egg",
            "--dist-dir",
            dist_dir,
        ],
        cwd=egg_dir,
        check=True,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
    )


class EggBasedTestCase(unittest.TestCase):
    """ Base class for Egg-based test cases. """

    @classmethod
    def setUpClass(cls):
        """
        Create eggs for testing purposes.
        """
        cls.egg_dir = tempfile.mkdtemp()
        eggs_root_dir = pkg_resources.resource_filename(
            "envisage.tests", "eggs")
        for egg_name in ["acme.bar", "acme.baz", "acme.foo"]:
            build_egg(
                egg_dir=join(eggs_root_dir, egg_name),
                dist_dir=cls.egg_dir,
            )

    @classmethod
    def tearDownClass(cls):
        """
        Delete created eggs.
        """
        shutil.rmtree(cls.egg_dir)

    def setUp(self):
        """ Prepares the test fixture before each test method is called. """

        # Some tests cause sys.path to be modified. Capture the original
        # contents so that we can restore sys.path later.
        self._original_sys_path_contents = sys.path[:]

    def tearDown(self):
        """ Called immediately after each test method has been called. """

        # Undo any sys.path modifications
        sys.path[:] = self._original_sys_path_contents

        pkg_resources.working_set = pkg_resources.WorkingSet()

    def _add_egg(self, filename, working_set=None):
        """ Create and add a distribution from the specified '.egg'. """

        if working_set is None:
            working_set = pkg_resources.working_set

        # The eggs must be in our egg directory!
        filename = join(self.egg_dir, filename)

        # Create a distribution for the egg.
        distributions = pkg_resources.find_distributions(filename)

        # Add the distributions to the working set (this makes any Python
        # modules in the eggs available for importing).
        for distribution in distributions:
            working_set.add(distribution)

    def _add_eggs_on_path(self, path, working_set=None):
        """ Add all eggs found on the path to a working set. """

        if working_set is None:
            working_set = pkg_resources.working_set

        environment = pkg_resources.Environment(path)

        # 'find_plugins' identifies those distributions that *could* be added
        # to the working set without version conflicts or missing requirements.
        distributions, errors = working_set.find_plugins(environment)
        # Py2 tests was checking that len(errors) > 0. This did not work on
        # Py3. Test changed to check the len(distributions)
        if len(distributions) == 0:
            raise SystemError("Cannot find eggs %s" % errors)

        # Add the distributions to the working set (this makes any Python
        # modules in the eggs available for importing).
        for distribution in distributions:
            working_set.add(distribution)



""" A view containing a red panel! """


# Local imports.
from color_view import ColorView


class RedView(ColorView):
    """ A view containing a red panel! """

    #### 'IView' interface ####################################################

    # The view's name.
    name = "Red"

    # The default position of the view relative to the item specified in the
    # 'relative_to' trait.
    position = "bottom"



""" Run the application.

Usually, all you have to do in here is:-

1) Initialise the logging package as you see fit (or not at all ;^)!

2) Set the 'EGG_PATH' variable to be a list of the directories that contain
   your application's eggs.

3) Edit the 'run' function to do whatever you need to do to start your
   application.

"""


# Standard library imports.
import logging


# Create a log file.
logger = logging.getLogger()
logger.addHandler(logging.StreamHandler(file('acmelab.log', 'w')))
logger.addHandler(logging.StreamHandler())
logger.setLevel(logging.DEBUG)


# A list of the directories that contain the application's eggs (any directory
# not specified as an absolute path is treated as being relative to the current
# working directory).
EGG_PATH = ['eggs']


def run():
    """ The function that starts your application. """

    # We do the imports here because the modules may well be in eggs that get
    # added to the path in 'main'.
    from envisage.api import EggPluginManager
    from acme.acmelab.api import Acmelab

    # Create an application that uses the egg plugin manager to find its
    # plugins.
    acmelab = Acmelab(plugin_manager=EggPluginManager())

    # Run it! This starts the application, starts the GUI event loop, and when
    # that terminates, stops the application.
    return acmelab.run()


###############################################################################
# Usually, there is no need to edit anything below here!
###############################################################################

# Standard library imports.
import logging
from pkg_resources import Environment, working_set


# Logging.
logger = logging.getLogger(__name__)


def main():
    """ Run the application. """

    # Find all additional eggs.
    environment = Environment(EGG_PATH)

    distributions, errors = working_set.find_plugins(environment)
    if len(errors) > 0:
        raise SystemError('cannot add eggs %s' % errors)

    logger.debug('added eggs %s' % distributions)

    # Add them to the working set.
    map(working_set.add, distributions)

    # Create and run the application.
    return run()


if __name__ == '__main__':
    main()

#### EOF ######################################################################



""" A test action set. """


# Enthought library imports.
from envisage.ui.action.api import Action, Group, Menu, ToolBar
from envisage.ui.workbench.api import WorkbenchActionSet


class TestActionSet(WorkbenchActionSet):
    """ An action test useful for testing. """

    #### 'ActionSet' interface ################################################

    # The action set's globally unique identifier.
    id = "envisage.ui.workbench.test"

    menus = [
        Menu(
            name="&Test",
            path="MenuBar",
            before="Help",
            groups=["XGroup", "YGroup"],
        ),
        Menu(name="Foo", path="MenuBar/Test", groups=["XGroup", "YGroup"]),
        Menu(name="Bar", path="MenuBar/Test", groups=["XGroup", "YGroup"]),
    ]

    groups = [Group(id="Fred", path="MenuBar/Test")]

    tool_bars = [
        ToolBar(name="Fred", groups=["AToolBarGroup"]),
        ToolBar(name="Wilma"),
        ToolBar(name="Barney"),
    ]

    actions = [
        Action(
            path="MenuBar/Test",
            group="Fred",
            class_name="envisage.ui.workbench.action.api:AboutAction",
        ),
        Action(
            path="MenuBar/Test",
            group="Fred",
            class_name="acme.workbench.action.new_view_action:NewViewAction",
        ),
        Action(
            path="ToolBar",
            class_name="envisage.ui.workbench.action.api:AboutAction",
        ),
        Action(
            path="ToolBar",
            class_name="envisage.ui.workbench.action.api:ExitAction",
        ),
        Action(
            path="ToolBar/Fred",
            group="AToolBarGroup",
            class_name="envisage.ui.workbench.action.api:AboutAction",
        ),
        Action(
            path="ToolBar/Wilma",
            class_name="envisage.ui.workbench.action.api:AboutAction",
        ),
        Action(
            path="ToolBar/Barney",
            class_name="envisage.ui.workbench.action.api:ExitAction",
        ),
    ]

    #### 'WorkbenchActionSet' interface #######################################

    # The Ids of the perspectives that the action set is enabled in.
    enabled_for_perspectives = ["Foo"]

    # The Ids of the perspectives that the action set is visible in.
    visible_for_perspectives = ["Foo", "Bar"]

    # The Ids of the views that the action set is enabled for.
    # enabled_for_views = ['Red']

    # The Ids of the views that the action set is visible for.
    # visible_for_views = ['Red']



# (C) Copyright 2007-2022 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!
# Major package imports.
from setuptools import setup, find_packages

setup(
    name="acme.bar",
    version="0.1a1",
    author="Enthought, Inc",
    author_email="info@enthought.com",
    license="BSD",
    zip_safe=True,
    packages=find_packages(),
    include_package_data=True,
    namespace_packages=["acme", "acme.bar"],
    install_requires=["acme.foo"],
    entry_points="""

    [envisage.plugins]
    acme.bar = acme.bar.bar_plugin:BarPlugin

    """,
)



# (C) Copyright 2007-2022 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!
""" Envisage package Copyright 2003, 2004, 2005 Enthought, Inc. """


from .workbench import Workbench
from .workbench_action_set import WorkbenchActionSet
from .workbench_application import WorkbenchApplication
from .workbench_window import WorkbenchWindow



# Major package imports.
from setuptools import setup, find_packages

setup(
    name="acme.motd.software_quotes",
    version="0.1a1",
    author="Enthought, Inc",
    author_email="info@enthought.com",
    license="BSD",
    zip_safe=True,
    packages=find_packages(),
    include_package_data=True,
    namespace_packages=["acme", "acme.motd", "acme.motd.software_quotes"],
    install_requires=["acme.motd"],
    entry_points="""

    [envisage.plugins]
    acme.motd.software_quotes = acme.motd.software_quotes.software_quotes_plugin:SoftwareQuotesPlugin

    """,
)



# Enthought library imports.
from traits.api import Enum, Str

# Local imports.
from attractors.model.i_model_2d import IModel2d


class IPlottable2d(IModel2d):

    plot_type = Enum("line", "scatter")

    x_label = Str()
    y_label = Str()



# Enthought library imports.
from pyface.tasks.api import (
    Task,
    TaskLayout,
    PaneItem,
    IEditor,
    IEditorAreaPane,
    SplitEditorAreaPane,
)
from pyface.tasks.action.api import (
    DockPaneToggleGroup,
    SMenuBar,
    SMenu,
    SToolBar,
    TaskAction,
)
from pyface.api import (
    ConfirmationDialog,
    FileDialog,
    ImageResource,
    YES,
    OK,
    CANCEL,
)
from traits.api import on_trait_change, Property, Instance

# Local imports.
from example_panes import PythonScriptBrowserPane
from python_editor import PythonEditor


class ExampleTask(Task):
    """ A simple task for editing Python code.
    """

    #### Task interface #######################################################

    id = "example.example_task"
    name = "Multi-Tab Editor"

    active_editor = Property(
        Instance(IEditor), observe="editor_area.active_editor"
    )

    editor_area = Instance(IEditorAreaPane)

    menu_bar = SMenuBar(
        SMenu(
            TaskAction(name="New", method="new", accelerator="Ctrl+N"),
            TaskAction(name="Open...", method="open", accelerator="Ctrl+O"),
            TaskAction(name="Save", method="save", accelerator="Ctrl+S"),
            id="File",
            name="&File",
        ),
        SMenu(DockPaneToggleGroup(), id="View", name="&View"),
    )

    tool_bars = [
        SToolBar(
            TaskAction(
                method="new",
                tooltip="New file",
                image=ImageResource("document_new"),
            ),
            TaskAction(
                method="open",
                tooltip="Open a file",
                image=ImageResource("document_open"),
            ),
            TaskAction(
                method="save",
                tooltip="Save the current file",
                image=ImageResource("document_save"),
            ),
            image_size=(32, 32),
        )
    ]

    ###########################################################################
    # 'Task' interface.
    ###########################################################################

    def _default_layout_default(self):
        return TaskLayout(left=PaneItem("example.python_script_browser_pane"))

    def activated(self):
        """ Overriden to set the window's title.
        """
        return
        filename = self.active_editor.path if self.active_editor else ""
        self.window.title = filename if filename else "Untitled"

    def create_central_pane(self):
        """ Create the central pane: the script editor.
        """
        self.editor_area = SplitEditorAreaPane()
        return self.editor_area

    def create_dock_panes(self):
        """ Create the file browser and connect to its double click event.
        """
        browser = PythonScriptBrowserPane()
        handler = lambda: self._open_file(browser.selected_file)
        browser.on_trait_change(handler, "activated")
        return [browser]

    ###########################################################################
    # 'ExampleTask' interface.
    ###########################################################################

    def new(self):
        """ Opens a new empty window
        """
        editor = PythonEditor()
        self.editor_area.add_editor(editor)
        self.editor_area.activate_editor(editor)
        self.activated()

    def open(self):
        """ Shows a dialog to open a file.
        """
        dialog = FileDialog(parent=self.window.control, wildcard="*.py")
        if dialog.open() == OK:
            self._open_file(dialog.path)

    def save(self):
        """ Attempts to save the current file, prompting for a path if
            necessary. Returns whether the file was saved.
        """
        editor = self.active_editor
        try:
            editor.save()
        except IOError:
            # If you are trying to save to a file that doesn't exist, open up a
            # FileDialog with a 'save as' action.
            dialog = FileDialog(
                parent=self.window.control, action="save as", wildcard="*.py"
            )
            if dialog.open() == OK:
                editor.save(dialog.path)
            else:
                return False
        return True

    ###########################################################################
    # Protected interface.
    ###########################################################################

    def _open_file(self, filename):
        """ Opens the file at the specified path in the editor.
        """
        editor = PythonEditor(path=filename)
        self.editor_area.add_editor(editor)
        self.editor_area.activate_editor(editor)
        self.activated()

    def _prompt_for_save(self):
        """ Prompts the user to save if necessary. Returns whether the dialog
            was cancelled.
        """
        dirty_editors = dict(
            [
                (editor.name, editor)
                for editor in self.editor_area.editors
                if editor.dirty
            ]
        )
        if not dirty_editors.keys():
            return True
        message = "You have unsaved files. Would you like to save them?"
        dialog = ConfirmationDialog(
            parent=self.window.control,
            message=message,
            cancel=True,
            default=CANCEL,
            title="Save Changes?",
        )
        result = dialog.open()
        if result == CANCEL:
            return False
        elif result == YES:
            for name, editor in dirty_editors.items():
                editor.save(editor.path)
        return True

    #### Trait change handlers ################################################

    @on_trait_change("window:closing")
    def _prompt_on_close(self, event):
        """ Prompt the user to save when exiting.
        """
        close = self._prompt_for_save()
        event.veto = not close

    #### Trait property getter/setters ########################################

    def _get_active_editor(self):
        if self.editor_area is not None:
            return self.editor_area.active_editor
        return None



""" A widget for editing Python code. """


# Enthought library imports.
from traits.api import Bool, Event, Instance, File, Str
from pyface.tasks.i_editor import IEditor

# Local imports.
from pyface.key_pressed_event import KeyPressedEvent


class IPythonEditor(IEditor):
    """ A widget for editing Python code. """

    #### 'IPythonEditor' interface ############################################

    # Object being editor is a file
    obj = Instance(File)

    # The pathname of the file being edited.
    path = Str

    # Should line numbers be shown in the margin?
    show_line_numbers = Bool(True)

    #### Events ####

    # The contents of the editor has changed.
    changed = Event

    # A key has been pressed.
    key_pressed = Event(KeyPressedEvent)

    ###########################################################################
    # 'IPythonEditor' interface.
    ###########################################################################

    def load(self, path=None):
        """ Loads the contents of the editor. """

    def save(self, path=None):
        """ Saves the contents of the editor. """

    def select_line(self, lineno):
        """ Selects the specified line. """



# Major package imports.
from setuptools import setup, find_packages

setup(
    name="acme.acmelab",
    version="0.1a1",
    author="Enthought, Inc",
    author_email="info@enthought.com",
    license="BSD",
    zip_safe=False,
    packages=find_packages(),
    include_package_data=True,
    namespace_packages=["acme", "acme.acmelab"],
)



""" Run the Lorenz example application. """


# Standard library imports.
import logging

# Enthought plugins.
from envisage.api import CorePlugin
from envisage.ui.workbench.workbench_plugin import WorkbenchPlugin

# Example imports.
from acme.lorenz.lorenz_application import LorenzApplication
from acme.lorenz.lorenz_plugin import LorenzPlugin
from acme.lorenz.lorenz_ui_plugin import LorenzUIPlugin


# Do whatever you want to do with log messages! Here we create a log file.
logger = logging.getLogger()
# logger.addHandler(logging.StreamHandler(file('lorenz.log', 'w')))
logger.addHandler(logging.StreamHandler())
logger.setLevel(logging.DEBUG)


def main():
    """ Run the application. """

    # Create an application with the specified plugins.
    lorenz_application = LorenzApplication(
        plugins=[
            CorePlugin(),
            WorkbenchPlugin(),
            LorenzPlugin(),
            LorenzUIPlugin(),
        ]
    )

    # Run it! This starts the application, starts the GUI event loop, and when
    # that terminates, stops the application.
    lorenz_application.run()

    return


if __name__ == "__main__":
    main()
